@@ -58,6 +58,7 @@ class TabularData(DataBase):
         categorical_columns: Optional[List[int]] = None,
         recognizer_configs: Optional[Dict[int, Dict[str, Any]]] = None,
         process_methods: Optional[Union[str, Dict[int, str]]] = "auto",
+        binning_method: str = "fuse",
         default_numerical_process: str = "normalize",
         default_categorical_process: str = "one_hot",
         use_timing_context: bool = True,
@@ -94,6 +95,7 @@ class TabularData(DataBase):
         self._recognizer_configs = recognizer_configs or {}
         self._label_recognizer_config = label_recognizer_config or {}
         self._process_methods = process_methods
+        self._binning_method = binning_method
         self._default_numerical_process = default_numerical_process
         self._default_categorical_process = default_categorical_process
         self._label_process_method = label_process_method
@@ -298,10 +300,11 @@ class TabularData(DataBase):
             is_label=True,
             is_valid=True,
             task_type=self._task_type,
+            binning=self._binning_method,
             config=self._label_recognizer_config,
         )
         assert self._raw is not None and self._raw.y is not None
-        recognizer.fit(self._y_df)
+        recognizer.fit(self._y_df, is_preset=False)
         return recognizer
 
     def _to_simplify_array(self, raw: DataTuple) -> DataTuple:
@@ -333,9 +336,22 @@ class TabularData(DataBase):
                 self._inject_label_recognizer()
         else:
             ts_indices = self.ts_indices
+            self.recognizers, self.converters = {}, {}
+            # convert labels
+            if self._raw is None or self._raw.y is None:
+                converted_labels = None
+                self.recognizers[-1] = None
+                self.converters[-1] = None
+            else:
+                with timing_context(self, "fit recognizer", enable=self._timing):
+                    recognizer = self._inject_label_recognizer()
+                with timing_context(self, "fit converter", enable=self._timing):
+                    converter = Converter.make_with(recognizer)
+                    self.converters[-1] = converter
+                with timing_context(self, "convert", enable=self._timing):
+                    converted_labels = converter.converted_input.reshape([-1, 1])
             # convert features
             converted_features = []
-            self.recognizers, self.converters = {}, {}
             if self._x_df is None:
                 raise ValueError("`_x_df` is required in `_core_fit`")
             for i in range(self.raw_dim):
@@ -360,10 +376,14 @@ class TabularData(DataBase):
                     recognizer = Recognizer(
                         column_name,
                         self._is_np,
+                        is_label=False,
                         is_valid=is_valid,
+                        task_type=self.task_type,
+                        binning=self._binning_method,
+                        labels=converted_labels,
                         config=recognizer_config,
                     )
-                    recognizer.fit(self._x_df[:, i])
+                    recognizer.fit(self._x_df[:, i], is_preset=i in self._preset_stypes)
                     self.recognizers[i] = recognizer
                 if not recognizer.info.is_valid:
                     self.log_msg(
@@ -381,19 +401,6 @@ class TabularData(DataBase):
                     with timing_context(self, "convert", enable=self._timing):
                         converted = converter.converted_input.astype(np_float_type)
                         converted_features.append(converted)
-            # convert labels
-            if self._raw is None or self._raw.y is None:
-                converted_labels = None
-                self.recognizers[-1] = None
-                self.converters[-1] = None
-            else:
-                with timing_context(self, "fit recognizer", enable=self._timing):
-                    recognizer = self._inject_label_recognizer()
-                with timing_context(self, "fit converter", enable=self._timing):
-                    converter = Converter.make_with(recognizer)
-                    self.converters[-1] = converter
-                with timing_context(self, "convert", enable=self._timing):
-                    converted_labels = converter.converted_input.reshape([-1, 1])
             converted_x = np.vstack(converted_features).T
             # process features
             self.processors = {}
@@ -508,6 +515,7 @@ class TabularData(DataBase):
                         f"specified label name ({self._label_name}) could not be found "
                         f"in the detected names ({names})"
                     )
+        self.label_indices = [self.label_idx]
         self.label_name = self.column_names[self.label_idx]
         # stypes
         stypes = {names[i]: v for i, v in self._preset_stypes.items()}
@@ -549,7 +557,15 @@ class TabularData(DataBase):
         # names
         self.label_name = self._label_name or "label"
         x_names = [f"C{i}" for i in range(len(x[0]))]
-        y_names = [self.label_name]
+        if y is None:
+            y_dim = 0
+            y_names = []
+        else:
+            y_dim = len(y[0]) if isinstance(y, list) else y.shape[1]
+            if y_dim == 1:
+                y_names = [self.label_name]
+            else:
+                y_names = [f"{self.label_name}_{i}" for i in range(y_dim)]
         names = x_names + y_names
         for i, name in (self._column_names or {}).items():
             names[i] = name
@@ -573,7 +589,9 @@ class TabularData(DataBase):
             y = y.astype(self._label_type.dtype)
         y_dt = to_dt_data(y)
         y_kwargs = {"names": y_names}
-        self.label_idx = len(self.column_names) - 1
+        all_dim = len(self.column_names)
+        self.label_idx = all_dim - y_dim
+        self.label_indices = list(range(self.label_idx, all_dim))
         # frames
         self._x_df = dt.Frame(x_dt, **x_kwargs)
         self._y_df = dt.Frame(y_dt, **y_kwargs)
@@ -652,7 +670,8 @@ class TabularData(DataBase):
         stypes = self._stypes.copy()
         names = [self.column_names[i] for i in range(len(self.column_names))]
         if not contains_labels:
-            names.pop(self.label_idx)
+            for idx in sorted(self.label_indices)[::-1]:
+                names.pop(idx)
             stypes.pop(self.label_name)
         return {"names": names, "stypes": stypes}
 
@@ -1037,6 +1056,7 @@ class TabularData(DataBase):
         if simplify:
             kwargs["simplify"] = simplify
         else:
+            kwargs.setdefault("binning_method", "identical")
             kwargs.setdefault("default_numerical_process", "identical")
             kwargs.setdefault("default_categorical_process", "identical")
         return cls(task_type=task_type, verbose_level=0, **kwargs)
