@@ -18,9 +18,9 @@ from cftool.misc import SavingMixin
 from cftool.misc import LoggingMixin
 from sklearn.utils import Bunch
 from sklearn.datasets import load_iris
-from sklearn.datasets import load_boston
 from sklearn.datasets import load_digits
 from sklearn.datasets import load_breast_cancer
+from sklearn.datasets import fetch_california_housing
 
 from ..types import *
 
@@ -41,7 +41,7 @@ def is_int(dtype: np.dtype) -> bool:
 
 
 def is_bool(dtype: np.dtype) -> bool:
-    return np.issubdtype(dtype, np.bool)
+    return np.issubdtype(dtype, np.bool_)
 
 
 def is_float(dtype: np.dtype) -> bool:
@@ -70,6 +70,13 @@ def to_dt_data(data: data_type) -> data_type:
     return transpose(data)
 
 
+def df_to_numpy(df: dt.Frame) -> np.ndarray:
+    try:
+        return df.to_numpy()
+    except:
+        return df.to_numpy(object)
+
+
 class DataTuple(NamedTuple):
     x: data_type
     y: data_type
@@ -147,8 +154,8 @@ class DataTuple(NamedTuple):
 
     @classmethod
     def from_dfs(cls, x_df: dt.Frame, y_df: Optional[dt.Frame]) -> "DataTuple":
-        x = x_df.to_numpy()
-        y = None if y_df is None else y_df.to_numpy()
+        x = df_to_numpy(x_df)
+        y = None if y_df is None else df_to_numpy(y_df)
         if isinstance(x, np.ma.core.MaskedArray):
             x = x.data
         if isinstance(y, np.ma.core.MaskedArray):
@@ -339,14 +346,14 @@ class TabularDataset(NamedTuple):
     def iris(cls) -> "TabularDataset":
         return cls.from_bunch(load_iris(), "clf")
 
-    @classmethod
-    def boston(cls) -> "TabularDataset":
-        return cls.from_bunch(load_boston(), "reg")
-
     @classmethod
     def digits(cls) -> "TabularDataset":
         return cls.from_bunch(load_digits(), "clf")
 
+    @classmethod
+    def california(cls) -> "TabularDataset":
+        return cls.from_bunch(fetch_california_housing(), "reg")
+
     @classmethod
     def breast_cancer(cls) -> "TabularDataset":
         return cls.from_bunch(load_breast_cancer(), "clf")
@@ -479,10 +486,10 @@ class TabularDataset(NamedTuple):
     ) -> Tuple["TabularDataset", "TabularDataset"]:
         assert p > 1, "p should be greater than 1"
         x_train = np.random.randn(size, n_dim)
-        x_train_list = [x_train] + [x_train ** i for i in range(2, p + 1)]
+        x_train_list = [x_train] + [x_train**i for i in range(2, p + 1)]
         x_train_noise = x_train + np.random.randn(size, n_dim) * noise_scale
         x_test = np.random.randn(int(size * test_ratio), n_dim)
-        x_test_list = [x_test] + [x_test ** i for i in range(2, p + 1)]
+        x_test_list = [x_test] + [x_test**i for i in range(2, p + 1)]
         idx_list = [np.random.permutation(n_dim)[:n_valid] for _ in range(p)]
         w_list = [np.random.randn(n_valid, 1) for _ in range(p)]
         o_train = [
